"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isDocCommand = exports.stringifyDoc = void 0;
const childProperties = [
    'breakContents',
    'contents',
    'flatContents',
    'parts',
];
function stringifyDoc(input, recursive = false) {
    if (typeof input === 'string' || !input) {
        return [
            input,
        ];
    }
    else if (Array.isArray(input)) {
        return input.map((entry) => stringifyDoc(entry, recursive));
    }
    else if (recursive) {
        const children = childProperties.reduce((accum, currentProperty) => {
            if (currentProperty in input) {
                accum.push(`${currentProperty}:`);
                accum.push(stringifyDoc(input[currentProperty], recursive));
            }
            return accum;
        }, []);
        if (children.length) {
            return [
                `${input.type}:`,
                children,
            ];
        }
    }
    return [
        input.type,
    ];
}
exports.stringifyDoc = stringifyDoc;
function isDocCommand(inputDoc) {
    return !!inputDoc && typeof inputDoc !== 'string' && !Array.isArray(inputDoc);
}
exports.isDocCommand = isDocCommand;
