"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addCustomPreprocessing = void 0;
const plugin_marker_1 = require("./plugin-marker");
const multiline_array_printer_1 = require("./printer/multiline-array-printer");
const original_printer_1 = require("./printer/original-printer");
function addMultilinePrinter(options) {
    var _a;
    if ('printer' in options) {
        (0, original_printer_1.setOriginalPrinter)(options.printer);
        // overwrite the printer with ours
        options.printer = multiline_array_printer_1.multilineArrayPrinter;
    }
    else {
        const astFormat = options.astFormat;
        if (!astFormat) {
            throw new Error(`Could not find astFormat while adding printer.`);
        }
        /** If the printer hasn't already been assigned in options, rearrange plugins so that ours gets chosen. */
        const plugins = options.plugins;
        const firstMatchedPlugin = plugins.find((plugin) => typeof plugin !== 'string' && !!plugin.printers && !!plugin.printers[astFormat]);
        if (!firstMatchedPlugin || typeof firstMatchedPlugin === 'string') {
            throw new Error(`Matched invalid first plugin: ${firstMatchedPlugin}`);
        }
        const matchedPrinter = (_a = firstMatchedPlugin.printers) === null || _a === void 0 ? void 0 : _a[astFormat];
        if (!matchedPrinter) {
            throw new Error(`Printer not found on matched plugin: ${firstMatchedPlugin}`);
        }
        (0, original_printer_1.setOriginalPrinter)(matchedPrinter);
        const thisPluginIndex = plugins.findIndex((plugin) => {
            return plugin.pluginMarker === plugin_marker_1.pluginMarker;
        });
        const thisPlugin = plugins[thisPluginIndex];
        if (!thisPlugin) {
            throw new Error(`This plugin was not found.`);
        }
        // remove this plugin from its current location in the array
        plugins.splice(thisPluginIndex, 1);
        // add this plugin to the beginning of the array so its printer is found first
        plugins.splice(0, 0, thisPlugin);
    }
}
function findPluginsByParserName(parserName, options) {
    return options.plugins.filter((plugin) => {
        var _a;
        return (typeof plugin === 'object' &&
            plugin.pluginMarker !== plugin_marker_1.pluginMarker &&
            !!((_a = plugin.parsers) === null || _a === void 0 ? void 0 : _a[parserName]));
    });
}
function addCustomPreprocessing(originalParser, parserName) {
    const thisPluginPreprocess = (text, options) => {
        const pluginsWithPreprocessor = findPluginsByParserName(parserName, options).filter((plugin) => { var _a, _b; return !!((_b = (_a = plugin.parsers) === null || _a === void 0 ? void 0 : _a[parserName]) === null || _b === void 0 ? void 0 : _b.preprocess); });
        let processedText = text;
        pluginsWithPreprocessor.forEach((pluginWithPreprocessor) => {
            var _a, _b, _c;
            const nextText = (_c = (_b = (_a = pluginWithPreprocessor.parsers) === null || _a === void 0 ? void 0 : _a[parserName]) === null || _b === void 0 ? void 0 : _b.preprocess) === null || _c === void 0 ? void 0 : _c.call(_b, processedText, {
                ...options,
                plugins: options.plugins.filter((plugin) => plugin.pluginMarker !== plugin_marker_1.pluginMarker),
            });
            if (nextText != undefined) {
                processedText = nextText;
            }
        });
        addMultilinePrinter(options);
        return processedText;
    };
    const parser = {
        ...originalParser,
        preprocess: thisPluginPreprocess,
    };
    return parser;
}
exports.addCustomPreprocessing = addCustomPreprocessing;
